"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stateful = exports.StatefulLifetime = void 0;
var ServiceProvider_1 = require("../ServiceProvider");
var ILifetime_1 = require("./ILifetime");
var StatefulLifetime = /** @class */ (function () {
    function StatefulLifetime(name, factory) {
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "factory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "last", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "next", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.name = name;
        this.factory = factory;
        this.last = ILifetime_1.Lifetime.dummy("".concat(name, "@constructor"));
        this.next = function (_a) {
            var instances = _a.instances;
            instances[name] = instances[name] || 1;
            return instances[name]++;
        };
    }
    Object.defineProperty(StatefulLifetime.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (context) {
            var _this = this;
            var _a;
            var parentContext = context.parent;
            var _b = (_a = context.lastSingleton) !== null && _a !== void 0 ? _a : {}, isSingleton = _b.isSingleton, lastName = _b.name;
            var singleton = lastName ? "".concat(String(lastName), "@") : '';
            var escapedContext = new ServiceProvider_1.ScopedServiceProvider(context.root)
                .enter(parentContext.depth && this.last)
                .enter(ILifetime_1.Lifetime.dummy("".concat(singleton).concat(this.name, "#").concat(this.next(context)), isSingleton));
            var trappedContext = parentContext.enter(ILifetime_1.Lifetime.dummy("".concat(this.name, "#")));
            return {
                create: function (props) {
                    var context = parentContext.isDone || !parentContext.depth ? escapedContext : trappedContext;
                    return _this.factory(context.proxy, props, context);
                },
            };
        }
    });
    Object.defineProperty(StatefulLifetime.prototype, "clone", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return new StatefulLifetime(this.name, this.factory);
        }
    });
    return StatefulLifetime;
}());
exports.StatefulLifetime = StatefulLifetime;
function stateful(dep) {
    return (typeof dep === 'function'
        ? { lifetime: StatefulLifetime, constructor: dep }
        : __assign({ lifetime: StatefulLifetime }, dep));
}
exports.stateful = stateful;
