import { ILifetime } from '../Lifetime';
import { ScopedServiceProvider } from '../ServiceProvider';
export declare type Key<E> = keyof E & (string);
export declare type LifetimeConstructor<T = any, P = void, E = any> = {
    new (name: Key<E>, factory: DependencyFactory<T, P, E>): ILifetime<T, E>;
};
export declare type LifetimeCollection<E = any> = {
    [key in keyof E]: ILifetime<unknown, E>;
};
export declare type MatchingProperties<T, E> = {
    [K in keyof E]: E[K] extends T ? K : never;
}[keyof E];
export declare type SelectorOptions<T = any, E = any> = {
    [key in MatchingProperties<T, E>]: key & Key<E>;
};
export declare type Selector<T, E> = Key<E> | ((e: SelectorOptions<T, E>) => Key<E>);
export declare type Stateful<P, T> = {
    create(props: P): T;
};
export declare type DependencyFactory<T, P, E> = (provider: E, props: P, scope: ScopedServiceProvider<E>) => T;
export declare type StatefulConstructor<T, P, E> = {
    new (provider: E, props: P): T;
};
export declare type NormalConstructor<T, E> = {
    new (provider: E): T;
} | {
    new (): T;
};
export declare type FactoryOption<T, P, E> = {
    factory: DependencyFactory<T, P, E>;
};
export declare type ConstructorOption<T, E> = {
    constructor: NormalConstructor<T, E>;
};
export declare type DependencyOption<T, E> = FactoryOption<T, void, E> | ConstructorOption<T, E>;
export declare type DependencyInformation<T, E> = {
    lifetime: LifetimeConstructor;
} & DependencyOption<T, E>;
export declare type DependencyMap<E, F> = {
    [key in keyof F]: DependencyInformation<F[key], any>;
};
